﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Threading;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Wpf_InMemoryFeatureLayerPerformanceSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private DispatcherTimer dispatcherTimer = null;

        public MainWindow()
        {
            InitializeComponent();

        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.Meter;

            // ImagePointStyle
            PointStyle pointImageStyle1 = new PointStyle(new GeoImage("../../App_Data/1.gif"));
            PointStyle pointImageStyle0 = new PointStyle(new GeoImage("../../App_Data/0.gif"));

            //// SymbolPointStyle
            //PointStyle pointImageStyle1 = PointStyles.CreateSimplePointStyle(PointSymbolType.Triangle,GeoColor.SimpleColors.Black, GeoColor.SimpleColors.Blue, 1, 20);
            //PointStyle pointImageStyle0 = PointStyles.CreateSimplePointStyle(PointSymbolType.Cross, GeoColor.SimpleColors.Black, GeoColor.SimpleColors.Blue, 1, 20);


            ClassBreakStyle c = new ClassBreakStyle("test");
            c.ClassBreaks.Add(new ClassBreak(0, pointImageStyle0));
            c.ClassBreaks.Add(new ClassBreak(1, pointImageStyle1));

            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.Open();
            inMemoryFeatureLayer.Columns.Add(new FeatureSourceColumn("test"));
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(c);
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            for (int i = 1; i <= 5000; i++)
            {
                Feature f = new Feature(i * 21, i * 25);
                f.ColumnValues.Add("test", "0");
                inMemoryFeatureLayer.InternalFeatures.Add(f);
            }
            // Build Index
            inMemoryFeatureLayer.BuildIndex();

            LayerOverlay layerOverlay = new LayerOverlay();
            layerOverlay.Layers.Add(inMemoryFeatureLayer);
            layerOverlay.TileType = TileType.SingleTile;
            layerOverlay.DrawingQuality = DrawingQuality.HighSpeed;
            wpfMap1.Overlays.Add(layerOverlay);

            wpfMap1.CurrentExtent = inMemoryFeatureLayer.GetBoundingBox();

            inMemoryFeatureLayer.Close();
            wpfMap1.Refresh();

            dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Tick += DispatcherTimer_Tick;
            dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 100);
            dispatcherTimer.Start();
        }

        private void DispatcherTimer_Tick(object sender, EventArgs e)
        {
            var layerOverlay = wpfMap1.Overlays[0] as LayerOverlay;
            var layer = layerOverlay.Layers[0] as InMemoryFeatureLayer;
            layer.Open();
            Stopwatch sw = new Stopwatch();
            sw.Start();
            var allFeature = layer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            layer.EditTools.BeginTransaction();

            for (int i = 1; i <= 1000; i++)
            {
                Feature target = allFeature[i];
                if (target.ColumnValues["test"] == "0")
                    target.ColumnValues["test"] = "1";
                else
                    target.ColumnValues["test"] = "0";
                layer.EditTools.Update(target);
            }

            layer.EditTools.CommitTransaction();
            sw.Stop();
            layerOverlay.Refresh();
        }
    }
}
